import 'dart:developer';
import 'package:aretmis_cupps_runner/global/MainModel.dart';
import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:get/get.dart';
import 'package:scoped_model/scoped_model.dart';

import '../main.dart';
import 'CuppsController.dart';
import 'data.dart';

class BpDeviceScreen extends StatefulWidget {
  final BpDevice device;

  const BpDeviceScreen({required this.device});

  @override
  _BpDeviceScreenState createState() => _BpDeviceScreenState();
}

class _BpDeviceScreenState extends State<BpDeviceScreen> {
  bool reInit = false;
  bool setAL = false;
  bool setLogo = false;
  bool setPectab = false;
  bool bpTest = false;
  bool release = false;

  @override
  Widget build(BuildContext context) {
    return ScopedModelDescendant<MainModel>(builder: (BuildContext context, Widget? child, MainModel model) {
      double width = MediaQuery.of(context).size.width;
      BpDevice device = widget.device;
      return Scaffold(
        appBar: AppBar(
          title: Text("${device.deviceIndex} ${device.deviceName}"),
          actions: [
            IconButton(
                onPressed: () async {
                  List<LogEntity> data = await Logger.getLogList();
                  String logDate = DateTime.now().toString().substring(0, 10);
                  List<LogEntity> logs = data
                      .where(
                          (d) => d.type.toLowerCase() == device.getType.name.toLowerCase().toLowerCase() && d.dateStr == logDate)
                      .toList();
                  if (logs.isNotEmpty) {
                    logs.first.download();
                  } else {
                    showGetToast("No Log Available", "No Log for today", color: Colors.orange);
                  }
                },
                icon: const Icon(Icons.bug_report_outlined))
          ],
        ),
        body: Column(
          children: [
            DeviceStatusMessageWidget(device: device),
            Expanded(
              child: Container(
                padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
                child: Column(
                  children: [
                    Row(
                      children: [
                        Expanded(flex: 1, child: Image.asset(device.icon, package: 'artemis_cupps')),
                        const SizedBox(width: 12),
                        Expanded(
                            flex: 2,
                            child: Column(
                              children: [
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    const Text("Status"),
                                    Text(
                                      "${device.deviceParameter.status}",
                                      style: const TextStyle(fontWeight: FontWeight.bold),
                                    ),
                                  ],
                                ),
                                const Divider(),
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    const Text("Address"),
                                    Text(
                                      "${device.deviceParameter.ipAndPort}",
                                      style: const TextStyle(fontWeight: FontWeight.bold),
                                    ),
                                  ],
                                ),
                                const Divider(),
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    GestureDetector(
                                        onTap: () {
                                          // device.connect().then((value){
                                          //   device.initialize();
                                          // });
                                        },
                                        child: const Text("TCP Connection")),
                                    Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red, size: 14)
                                  ],
                                ),
                              ],
                            ))
                      ],
                    ),
                    // Row(
                    //   children: [
                    //     const Text("Available Modes"),
                    //     const SizedBox(width: 12),
                    //     Expanded(
                    //       child: Wrap(
                    //         direction: Axis.horizontal,
                    //         children: device.supportedInterfaceModes.map((e) => OutlinedButton(onPressed: () {}, child: Text(e.mode))).toList(),
                    //       ),
                    //     ),
                    //   ],
                    // ),
                    const Divider(),
                    Expanded(
                      child: SingleChildScrollView(
                        child: Column(
                          children: [
                            ElevatedButton(
                              onPressed: () async {
                                // if(reInit) return;
                                setState(() {
                                  reInit = true;
                                });
                                final res = await device.restart().timeout(Duration(seconds: 12), onTimeout: () {
                                  if(device.initialized && device.modded) {
                                    return CuppsCommandResponse(status: true, msg: "Done");
                                  }else{
                                    return CuppsCommandResponse(status: false, msg: "TimeOut");
                                  }
                                });
                                setState(() {
                                  reInit = false;
                                });
                                showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                log("$res");
                              },
                              child: reInit
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Reset"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                if (setAL) return;
                                setState(() {
                                  setAL = true;
                                });
                                final res = await device.auth(airlineID: "001", font: "L");
                                setState(() {
                                  setAL = false;
                                });
                                showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                log("$res");
                              },
                              child: setAL
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("SetAL"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                if (setLogo) return;
                                setState(() {
                                  setLogo = true;
                                });
                                final res = await device.setLogo(DataExample.logoText);
                                setState(() {
                                  setLogo = false;
                                });
                                showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                log("$res");
                              },
                              child: setLogo
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Set Logo"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                if (setPectab) return;
                                setState(() {
                                  setPectab = true;
                                });
                                final res = await device.setPectab(DataExample.pectabText);
                                setState(() {
                                  setPectab = false;
                                });
                                showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                log("$res");
                              },
                              child: setPectab
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Set Pectab"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                if (bpTest) return;
                                setState(() {
                                  bpTest = true;
                                });
                                // final res = await device.print(DataExample.pectabTextBT);
                                final res = await CuppsController(model).printBps(DataExample.commandBpBt.prints.firstWhere((element) =>
                                element.type==0).data);
                                setState(() {
                                  bpTest = false;
                                });
                                // showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                // log("$res");
                              },
                              child: bpTest
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Print Bp Test"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                if (release) return;
                                setState(() {
                                  release = true;
                                });
                                final res = await device.release();
                                setState(() {
                                  release = false;
                                });
                                showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                                log("$res");
                              },
                              child: release
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Release"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                device.refreshStatus();
                              },
                              child: release
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Refresh Status"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                final req =  InterfaceLevelRequest(messageID: 100, level: '01.03');
                                device.send(xml: req.toXML());
                              },
                              child: release
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Session Error"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                final req =  DeviceStatusRequest(messageID: 100);
                                device.send(xml: req.toXML());
                              },
                              child: release
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Message ID Error"),
                                      ],
                                    ),
                            ),
                            ElevatedButton(
                              onPressed: () async {
                                List<String> confComs =
                                    model.printInitializers.prints.firstWhere((element) => element.type == 0).data;
                                await model.platformDevices.bpDevices.first.completeConfigure(confComs);
                                // for (String commmand in confComs) {
                                //   await model.platformDevices.bpDevices.first.configure(commmand);
                                // }
                              },
                              child: release
                                  ? SpinKitThreeBounce(color: Colors.white, size: 20)
                                  : Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      children: [
                                        const Text("Configure"),
                                      ],
                                    ),
                            ),
                          ],
                        ),
                      ),
                    )
                  ],
                ),
              ),
            ),
          ],
        ),
      );
    });
  }
}
