import 'dart:developer';

import 'package:aretmis_cupps_runner/global/MainModel.dart';
import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:get/get.dart';
import 'package:scoped_model/scoped_model.dart';

import '../main.dart';
import 'data.dart';

class PrDeviceScreen extends StatefulWidget {
  final PrDevice device;

  const PrDeviceScreen({required this.device});

  @override
  _PrDeviceScreenState createState() => _PrDeviceScreenState();
}

class _PrDeviceScreenState extends State<PrDeviceScreen> {
  bool reInit = false;
  bool release = false;
  bool lockUnlock = false;
  bool printing = false;

  @override
  Widget build(BuildContext context) {
    return ScopedModelDescendant<MainModel>(builder: (BuildContext context, Widget? child, MainModel model) {
      double width = MediaQuery.of(context).size.width;
      PrDevice device = widget.device;
      return Scaffold(
        appBar: AppBar(
          title: Text("${device.deviceIndex} ${device.deviceName}"),
          actions: [
            IconButton(
                onPressed: () async {
                  List<LogEntity> data = await Logger.getLogList();
                  String logDate = DateTime.now().toString().substring(0, 10);
                  List<LogEntity> logs = data
                      .where(
                          (d) => d.type.toLowerCase() == device.getType.name.toLowerCase().toLowerCase() && d.dateStr == logDate)
                      .toList();
                  if(logs.isNotEmpty){
                    logs.first.download();
                  }else{
                    showGetToast("No Log Available","No Log for today",color: Colors.orange);
                  }

                },
                icon: const Icon(Icons.bug_report_outlined))
          ],
        ),
        body: Column(
          children: [
            DeviceStatusMessageWidget(device: device),
            Expanded(
              child: Container(
                padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
                child: Column(
                  children: [
                    Row(
                      children: [
                        Expanded(flex: 1, child: Image.asset(device.icon, package: 'artemis_cupps')),
                        const SizedBox(width: 12),
                        Expanded(
                            flex: 2,
                            child: Column(
                              children: [
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    const Text("Status"),
                                    Text(
                                      "${device.deviceParameter.status}",
                                      style: const TextStyle(fontWeight: FontWeight.bold),
                                    ),
                                  ],
                                ),
                                const Divider(),
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    const Text("Address"),
                                    Text(
                                      "${device.deviceParameter.ipAndPort}",
                                      style: const TextStyle(fontWeight: FontWeight.bold),
                                    ),
                                  ],
                                ),
                                const Divider(),
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    const Text("TCP Connection"),
                                    Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red, size: 14)
                                  ],
                                ),
                              ],
                            ))
                      ],
                    ),
                    const Divider(),
                    ElevatedButton(
                      onPressed: () async {
                        // if(reInit) return;
                        setState(() {
                          reInit = true;
                        });
                        final res = await device.restart().timeout(Duration(seconds: 12), onTimeout: () {
                          return CuppsCommandResponse(status: false, msg: "TimeOut");
                        });
                        setState(() {
                          reInit = false;
                        });
                        showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                        log("$res");
                      },
                      child: reInit
                          ? SpinKitThreeBounce(color: Colors.white, size: 20)
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                const Text("Reset"),
                              ],
                            ),
                    ),
                    ElevatedButton(
                      onPressed: () async {
                        if (lockUnlock) return;
                        setState(() {
                          lockUnlock = true;
                        });
                        if (device.locked) {
                          final res = await device.unlock();
                          setState(() {
                            lockUnlock = false;
                          });
                          showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                          log("$res");
                        } else {
                          final res = await device.lock();
                          setState(() {
                            lockUnlock = false;
                          });
                          showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                          log("$res");
                        }
                      },
                      child: lockUnlock
                          ? SpinKitThreeBounce(color: Colors.white, size: 20)
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Text("${device.locked ? "Unlock" : "Lock"}"),
                              ],
                            ),
                    ),
                    ElevatedButton(
                      onPressed: () async {
                        if (printing) return;
                        setState(() {
                          printing = true;
                        });
                        final res = await device.printPr(DataExample.prText);
                        setState(() {
                          printing = false;
                        });
                        showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                        log("$res");
                      },
                      child: printing
                          ? SpinKitThreeBounce(color: Colors.white, size: 20)
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Text("Print Document"),
                              ],
                            ),
                    ),
                    ElevatedButton(
                      onPressed: () async {
                        if (release) return;
                        setState(() {
                          release = true;
                        });
                        final res = await device.release();
                        setState(() {
                          release = false;
                        });
                        showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
                        log("$res");
                      },
                      child: release
                          ? SpinKitThreeBounce(color: Colors.white, size: 20)
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                const Text("Release"),
                              ],
                            ),
                    ),
                    ElevatedButton(
                      onPressed: () async {
                        device.refreshStatus();
                      },
                      child: release
                          ? SpinKitThreeBounce(color: Colors.white, size: 20)
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                const Text("Refresh Status"),
                              ],
                            ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
      );
    });
  }
}
